@@ -171,7 +171,7 @@ class GenericAuthorizationRole(Role):
     name_suffix = 'user_based_crud_role'
 
     def get_name(self):
-        return f"{self.collection.get_class_name()}_{self.name_suffix}"
+        return self.name or f"{self.collection.get_class_name()}_{self.name_suffix}"
 
     def get_privileges(self):
         return [
@@ -196,6 +196,12 @@ class GenericAuthorizationRole(Role):
                     "read": True
                 }
             },
+            {
+                "resource": q.function(self.collection.all_function_name()),
+                "actions": {
+                    "call": True
+                }
+            },
             {
                 "resource": q.function(f'create_{self.collection.get_class_name()}'),
                 "actions": {
@@ -219,7 +225,7 @@ class GenericAuthorizationRole(Role):
 
 class GenericUserBasedRole(GenericAuthorizationRole):
     relation_index_name = 'users_groups_by_user'
-    
+
     def get_lambda(self, resource_type):
         if resource_type == 'write':
             lambda_args = ["old_object", "new_object", "object_ref"]
@@ -320,4 +326,3 @@ class GenericGroupBasedRole(GenericAuthorizationRole):
                                 )
                             )
                         )
-
